/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://savefileto/common.jsm");

const dirSvc = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
const promptSvc = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);

const DEFAULT_DIRECTORY = "Desk";


var dialog =
{
    get args()        { return window.arguments[0].args; },
    set result(value) { window.arguments[0].result = value; },
    
    get label() { return document.getElementById("tbox_label"); },
    get path()  { return document.getElementById("tbox_path"); },
    get menu()  { return document.getElementById("cbox_menu"); },
    
    get file()
    {
        try
        {
            var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
            file.initWithPath(this.path.value);
            return file;
        }
        catch (e)
        {
            return null;
        }
    },
    
    onLoad: function()
    {
        this.bundle = document.getElementById("bundle");
        
        document.title = this.bundle.getString("title." + this.args.title);
        
        this.label.value = this.args.label;
        this.path.value = this.args.path;
        this.menu.checked = this.args.menu;
    },
    
    pickFolder: function()
    {
        var filePicker = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
        
        var file = this.file;
        if (! (file && file.exists() && file.isDirectory()))
            file = getNamedDirectory(DEFAULT_DIRECTORY);
        filePicker.displayDirectory = file;
        
        var title = this.bundle.getString("folderPickerTitle");
        var mode = Ci.nsIFilePicker.modeGetFolder;
        filePicker.init(window, title, mode);
        var value = filePicker.show();
        
        if (value == Ci.nsIFilePicker.returnOK)
        {
            if (this.label.value.trim().length == 0)
                this.label.value = filePicker.file.leafName;
            this.path.value = filePicker.file.path;
        }
    },
    
    onDialogAccept: function()
    {
        try
        {
            var label = this.label.value;
            if (/^\s*$/.test(label))
                throw "error.emptyName";
            
            var path = this.path.value;
            if (/^\s*$/.test(path))
                throw "error.emptyPath";
            if (! this.file)
                throw "error.folderNotFound";
            
            var menu = this.menu.checked;
            
            this.result = {label: label, path: path, menu: menu};
            return true;
        }
        catch (e if (typeof(e) == "string"))
        {
            var title = this.bundle.getString("error.title");
            var message = this.bundle.getString(e);
            promptSvc.alert(window, title, message);
            return false;
        }
    }
}
